from pathlib import Path
import click
from click import command, option, argument
from lager import log
from mrender.md import Markdown
import sys
import importlib.util
import requests
from requests.exceptions import RequestException
import os

from mrender.md import recursive_read
from mrender.docs2md import generate_docs
from mrender.web2md import html_to_markdown_with_depth, extract_links
from mrender.web2md import display_markdown_hierarchically

@command(context_settings=dict(help_option_names=['-h', '--help']))
@argument("input", required=False)
@option("--output", "-o", type=click.Path(), help="Output file")
@option("--format", "-f", default="md", help="Output format")
@option("--depth", "-d", default=3, type=int, help="Depth of the output")
@option("--docs", is_flag=True, help="Generate documentation")
@option("--web", is_flag=True, help="Process input as a web URL")
def cli(input: str = None, output: str = None, format: str = "md", depth: int = 3, docs: bool = False, web: bool = False):
    """
    Process input file, directory, or web URL and generate output.

    If no input is provided, this help message will be displayed.
    If --web is specified or the input starts with http:// or https://, the input is treated as a web URL.
    """
    if not input:
        click.echo(click.get_current_context().get_help())
        return

    try:
        log.info(f"Processing input: {input}")
        
        if web or input.startswith(('http://', 'https://')):
            response = requests.get(input)
            response.raise_for_status()
            html_content = response.text
            markdown_content = html_to_markdown_with_depth(html_content, depth)
            links = extract_links(markdown_content)
            if output:
                Path(output).write_text(markdown_content)
                click.echo(f"Web content saved to {output}")
            else:
                display_markdown_hierarchically(markdown_content, depth)
            return

        if docs:
            if not Path(input).exists():
                raise click.ClickException(f"Invalid input: {input}. Path does not exist.")
            
            if Path(input).is_file():
                markdown_content = generate_docs(input)
            elif Path(input).is_dir():
                markdown_content = ""
                for file in Path(input).glob('**/*.py'):
                    markdown_content += generate_docs(str(file)) + "\n\n"
            else:
                raise click.ClickException(f"Invalid input: {input}. Must be a file or directory.")
            
            if output:
                Path(output).write_text(markdown_content)
                click.echo(f"Documentation saved to {output}")
            else:
                click.echo(markdown_content)
            return

        if not Path(input).exists():
            raise click.ClickException(f"Invalid input: {input}. Must be a valid file path or directory.")

        data = recursive_read(input)
        if format == "md":
            md = Markdown(data=data, save=output)
            md.stream(depth=depth)

    except requests.exceptions.RequestException as e:
        raise click.ClickException(f"Error fetching web content: {str(e)}")
    except click.ClickException:
        raise
    except Exception as e:
        raise click.ClickException(str(e))

if __name__ == '__main__':
    cli()
